import 'package:animated_custom_dropdown/custom_dropdown.dart';
import 'package:flutter/material.dart';

import 'scale_theme.dart';

class ScaleCustomDropdownTheme {
  ScaleCustomDropdownTheme({
    required this.decoration,
    required this.closedHeaderPadding,
    required this.expandedHeaderPadding,
    required this.itemsListPadding,
    required this.listItemPadding,
    required this.disabledDecoration,
    required this.textStyle,
  });

  final CustomDropdownDecoration decoration;
  final EdgeInsets closedHeaderPadding;
  final EdgeInsets expandedHeaderPadding;
  final EdgeInsets itemsListPadding;
  final EdgeInsets listItemPadding;
  final CustomDropdownDisabledDecoration disabledDecoration;
  final TextStyle textStyle;
}

extension ScaleCustomDropdownThemeExt on ScaleTheme {
  ScaleCustomDropdownTheme customDropdownTheme() {
    final scale = scheme.primaryScale;
    final borderColor = scale.borderText;
    final fillColor = scale.subtleBorder;

    // final backgroundColor = config.useVisualIndicators && !selected
    //     ? tileColor.borderText
    //     : borderColor;
    // final textColor = config.useVisualIndicators && !selected
    //     ? borderColor
    //     : tileColor.borderText;

    // final largeTextStyle = textTheme.labelMedium!.copyWith(color: textColor);
    // final smallTextStyle = textTheme.labelSmall!.copyWith(color: textColor);

    final border = Border.fromBorderSide(config.useVisualIndicators
        ? BorderSide(width: 2, color: borderColor, strokeAlign: 0)
        : BorderSide.none);
    final borderRadius = BorderRadius.circular(8 * config.borderRadiusScale);

    final decoration = CustomDropdownDecoration(
      closedFillColor: fillColor,
      expandedFillColor: fillColor,
      closedShadow: [],
      expandedShadow: [],
      closedSuffixIcon: Icon(Icons.arrow_drop_down, color: borderColor),
      expandedSuffixIcon: Icon(Icons.arrow_drop_up, color: borderColor),
      prefixIcon: null,
      closedBorder: border,
      closedBorderRadius: borderRadius,
      closedErrorBorder: null,
      closedErrorBorderRadius: null,
      expandedBorder: border,
      expandedBorderRadius: borderRadius,
      hintStyle: null,
      headerStyle: null,
      noResultFoundStyle: null,
      errorStyle: null,
      listItemStyle: null,
      overlayScrollbarDecoration: null,
      searchFieldDecoration: null,
      listItemDecoration: null,
    );

    final disabledDecoration = CustomDropdownDisabledDecoration(
      fillColor: null,
      shadow: null,
      suffixIcon: null,
      prefixIcon: null,
      border: null,
      borderRadius: null,
      headerStyle: null,
      hintStyle: null,
    );

    return ScaleCustomDropdownTheme(
      textStyle: textTheme.labelSmall!.copyWith(color: borderColor),
      decoration: decoration,
      closedHeaderPadding: const EdgeInsets.all(4),
      expandedHeaderPadding: const EdgeInsets.all(4),
      itemsListPadding: const EdgeInsets.all(4),
      listItemPadding: const EdgeInsets.all(4),
      disabledDecoration: disabledDecoration,
    );
  }
}
